import {
  Inject,
  Injectable,
  NestMiddleware,
  Logger,
  LoggerService,
} from '@nestjs/common';
import { Request, Response, NextFunction } from 'express';

@Injectable()
export default class LoggerMiddleware implements NestMiddleware {
  // TODO: Access log if env not production
  constructor(@Inject(Logger) private readonly logger: LoggerService) {}

  use(req: Request, res: Response, next: NextFunction) {
    if (process.env.NODE_ENV === 'production') {
      const logObj = {
        message: 'web-request',
        request: this.requestInJson(req),
      };
      if (process.env.LOG_LEVEL_INFO_FOR_WEB_REQUEST === 'true') {
        this.logger.log(logObj, LoggerMiddleware.name);
      } else {
        this.logger.debug(logObj, LoggerMiddleware.name);
      }
    } else {
      const logMsg = this.requestInLine(req);
      if (process.env.LOG_LEVEL_INFO_FOR_WEB_REQUEST === 'true') {
        this.logger.log(logMsg, LoggerMiddleware.name);
      } else {
        this.logger.debug(logMsg, LoggerMiddleware.name);
      }
    }

    next();
  }

  requestInLine(req: Request): string {
    const { headers, url, method, body, socket } = req;
    const xRealIp = headers['X-Real-IP'];
    const xForwardedFor = headers['X-Forwarded-For'];
    const { ip: cIp } = req;
    const { remoteAddress } = socket || {};
    const ip = xRealIp || xForwardedFor || cIp || remoteAddress;

    const requstBody =
      Object.keys(body).length > 0 ? ` Body=${JSON.stringify(body)}` : '';
    const requstIp = ip ? ` SrcIP=${ip}` : '';

    return `${method.toLocaleUpperCase()} ${url}${requstBody}${requstIp}`;
  }

  requestInJson: (req: Request) => {
    [prop: string]: any;
  } = (req) => {
    const { headers, url, method, body, socket } = req;
    const xRealIp = headers['X-Real-IP'];
    const xForwardedFor = headers['X-Forwarded-For'];
    const { ip: cIp } = req;
    const { remoteAddress } = socket || {};
    const ip = xRealIp || xForwardedFor || cIp || remoteAddress;

    return {
      url,
      host: headers.host,
      ip,
      method,
      body,
    };
  };
}
